# Generated by Django 3.2.3 on 2021-05-26 03:32

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("example", "0008_labresults"),
    ]

    operations = [
        migrations.AddField(
            model_name="labresults",
            name="author",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="lab_results",
                to="example.author",
            ),
        ),
    ]
