# Generated by Django 3.0.9 on 2021-07-14 08:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("example", "0009_labresults_author"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="labresults",
            options={"ordering": ("id",)},
        ),
    ]
