# Generated by Django 3.0.3 on 2020-02-06 10:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("example", "0007_artproject_description"),
    ]

    operations = [
        migrations.CreateModel(
            name="LabResults",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField()),
                ("measurements", models.TextField()),
                (
                    "research_project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="lab_results",
                        to="example.ResearchProject",
                    ),
                ),
            ],
        ),
    ]
